#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_fs_1 = __importDefault(require("node:fs"));
const node_path_1 = __importDefault(require("node:path"));
const yargs_1 = __importDefault(require("yargs"));
const helpers_1 = require("yargs/helpers");
const chalk_1 = __importDefault(require("chalk"));
const cli_spinners_1 = __importDefault(require("cli-spinners"));
const log_update_1 = __importDefault(require("log-update"));
const prompts_1 = __importDefault(require("prompts"));
const file_writers_1 = require("./templates/file-writers");
const spectaclePackage = require(`${__dirname}/../spectacle-package.json`);
const argv = (0, yargs_1.default)((0, helpers_1.hideBin)(process.argv)).argv;
const cwd = process.cwd();
var ArgName;
(function (ArgName) {
    ArgName["type"] = "type";
    ArgName["name"] = "name";
    ArgName["lang"] = "lang";
    ArgName["port"] = "port";
    ArgName["overwrite"] = "overwrite";
})(ArgName || (ArgName = {}));
const DeckTypeOptions = [
    { title: 'One Page', value: 'onepage' },
    { title: 'Markdown', value: 'md' },
    { title: 'React using Vite', value: 'tsx-vite' },
    { title: 'React using webpack', value: 'tsx' }
];
let progressInterval;
const log = console.log;
const printConsoleError = (message) => chalk_1.default.whiteBright.bgRed.bold(' ! ') + chalk_1.default.red.bold(' ' + message + '\n');
const sleep = (ms) => new Promise((resolve) => setTimeout(resolve, ms));
const main = async () => {
    log(chalk_1.default.whiteBright.bgMagenta.bold(' Spectacle CLI '));
    let i = 0;
    let type = argv[ArgName.type] || argv['t'];
    let name = argv['_']?.[0];
    let lang = argv[ArgName.lang] || argv['l'];
    let port = argv[ArgName.port] || argv['p'];
    const isTryingToOverwrite = Boolean(name) && !isOutputPathAvailable(name, type === 'onepage');
    /**
     * If type/name not both provided via CLI flags, prompt for them.
     */
    const hasType = Boolean(type);
    const hasName = Boolean(name);
    const hasLang = Boolean(lang);
    const hasPort = type === 'onepage' || Boolean(port); // onepage has no port
    if (!(hasType && hasName && hasLang && hasPort) || isTryingToOverwrite) {
        try {
            const response = await (0, prompts_1.default)([
                // Name prompt
                {
                    type: 'text',
                    name: ArgName.name,
                    message: 'What is the name of the presentation?',
                    initial: name,
                    validate: async (val) => {
                        return val.trim().length > 0 ? true : 'Name is required';
                    }
                },
                /**
                 * Type of deck.
                 * Needs to be before overwrite confirmation so we can determine if folder/file already exists.
                 */
                {
                    type: 'select',
                    name: ArgName.type,
                    message: 'What type of deck do you want to create?',
                    choices: DeckTypeOptions,
                    initial: (() => {
                        const ind = DeckTypeOptions.findIndex((o) => o.value === type);
                        return ind > -1 ? ind : 0;
                    })()
                },
                // If output directory already exists, prompt to overwrite
                {
                    type: (_, answers) => isOutputPathAvailable(answers?.[ArgName.name], answers?.[ArgName.type] === 'onepage')
                        ? null
                        : 'confirm',
                    name: ArgName.overwrite,
                    message: (_, answers) => {
                        const name = answers?.[ArgName.name];
                        const type = answers?.[ArgName.type];
                        if (type === 'onepage') {
                            return `File ${formatProjectOutputPath(name)}.html already exists. Overwrite and continue?`;
                        }
                        else {
                            return `Target directory ${formatProjectOutputPath(name)} already exists. Overwrite and continue?`;
                        }
                    }
                },
                // Check overwrite comes back false, we need to abort.
                {
                    type: (_, answers) => {
                        if (answers?.[ArgName.overwrite] === false) {
                            throw new Error('❌ Operation cancelled');
                        }
                        return null;
                    },
                    name: 'overwriteAborter'
                },
                // Language prompt
                {
                    type: 'text',
                    name: ArgName.lang,
                    message: 'What is the language code for the generated HTML document?',
                    initial: lang || 'en',
                    validate: async (val) => {
                        return val.trim().length > 0 ? true : 'Language code is required';
                    }
                },
                {
                    // Don't prompt for this if onepage
                    type: (_, answers) => answers?.[ArgName.type] === 'onepage' ? null : 'text',
                    name: ArgName.port,
                    message: 'What port should the webpack dev server run on?',
                    initial: port || '3000'
                }
            ], {
                onCancel: () => {
                    throw new Error('❌ Operation cancelled');
                }
            });
            if (response.type)
                type = response.type;
            if (response.name)
                name = response.name;
            lang = response.lang;
            port = response.port;
        }
        catch (err) {
            console.log(chalk_1.default.red(err.message));
            return;
        }
    }
    progressInterval = setInterval(() => {
        const { frames } = cli_spinners_1.default.aesthetic;
        (0, log_update_1.default)(chalk_1.default.whiteBright.bgBlue.bold(` ${frames[(i = ++i % frames.length)]} `), chalk_1.default.blue.bold('Building Deck'));
    }, cli_spinners_1.default.dots.interval);
    await sleep(750);
    const fileOptions = {
        snakeCaseName: formatProjectOutputPath(name),
        name,
        lang,
        port,
        isVite: /vite$/.test(type),
        spectacleVersion: spectaclePackage.version
    };
    switch (type) {
        case 'md':
            await (0, file_writers_1.writeWebpackProjectFiles)({
                ...fileOptions,
                useMarkdownSlides: true
            });
            break;
        case 'tsx':
            await (0, file_writers_1.writeWebpackProjectFiles)(fileOptions);
            break;
        case 'tsx-vite':
            await (0, file_writers_1.writeViteProjectFiles)(fileOptions);
            break;
        case 'onepage':
            await (0, file_writers_1.writeOnePageHTMLFile)(fileOptions);
            break;
    }
    clearInterval(progressInterval);
    const atLocation = type === 'onepage'
        ? `at ${fileOptions.snakeCaseName}.html`
        : `in ${fileOptions.snakeCaseName}/`;
    (0, log_update_1.default)(chalk_1.default.whiteBright.bgGreen.bold(` \u2714 `), chalk_1.default.green.bold(`A new ${type} deck named ${name} was created ${atLocation}.\n`));
};
const formatProjectOutputPath = (name) => name.toLowerCase().replace(/([^a-z0-9]+)/gi, '-');
const isOutputPathAvailable = (name, isHTMLFile = false) => {
    const outputPath = isHTMLFile
        ? node_path_1.default.join(cwd, `${formatProjectOutputPath(name)}.html`)
        : node_path_1.default.join(cwd, formatProjectOutputPath(name));
    return !node_fs_1.default.existsSync(outputPath);
};
main().catch((err) => {
    clearInterval(progressInterval);
    (0, log_update_1.default)(printConsoleError(err.message));
});
