"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeOnePageHTMLFile = exports.writeViteProjectFiles = exports.writeWebpackProjectFiles = void 0;
const path_1 = __importDefault(require("path"));
const promises_1 = require("fs/promises");
const html_1 = require("./html");
const webpack_1 = require("./webpack");
const babel_1 = require("./babel");
const package_1 = require("./package");
const index_1 = require("./index");
const tsconfig_1 = require("./tsconfig");
const gitignore_1 = require("./gitignore");
const readme_1 = require("./readme");
const viteConfig_1 = require("./viteConfig");
const one_page_1 = require("../generators/one-page");
const markdown_1 = require("./markdown");
const prepForProjectWrite = async (fileOptions) => {
    const { name, lang, snakeCaseName, isVite } = fileOptions;
    const outPath = path_1.default.resolve(process.cwd(), snakeCaseName);
    const pathFor = (file) => path_1.default.join(outPath, file);
    // Clear out the directory if it already exists.
    await (0, promises_1.rm)(outPath, { recursive: true, force: true });
    // Make new directory, and add some base files (shared between webpack and vite).
    await (0, promises_1.mkdir)(outPath, { recursive: true });
    await (0, promises_1.writeFile)(pathFor('index.html'), (0, html_1.htmlTemplate)({
        name,
        lang,
        entryFile: isVite ? '/index.tsx' : undefined
    }));
    await (0, promises_1.writeFile)(pathFor('index.tsx'), (0, index_1.indexTemplate)({
        name,
        usesMarkdown: fileOptions.useMarkdownSlides
    }));
    await (0, promises_1.writeFile)(pathFor('.gitignore'), (0, gitignore_1.gitignoreTemplate)());
    await (0, promises_1.writeFile)(pathFor('README.md'), (0, readme_1.readmeTemplate)({ name, isVite }));
    await (0, promises_1.writeFile)(pathFor('tsconfig.json'), (0, tsconfig_1.tsconfigTemplate)());
    fileOptions.useMarkdownSlides &&
        (await Promise.all([
            (0, promises_1.writeFile)(pathFor('custom.d.ts'), (0, markdown_1.markdownCustomTypesTemplate)()),
            (0, promises_1.writeFile)(pathFor('slides.md'), (0, markdown_1.markdownTemplate)({ name }))
        ]));
    return { outPath, pathFor };
};
/**
 * Generate a webpack-based project
 */
const writeWebpackProjectFiles = async (options) => {
    const { port, snakeCaseName, spectacleVersion } = options;
    const { pathFor } = await prepForProjectWrite(options);
    await (0, promises_1.writeFile)(pathFor('webpack.config.js'), (0, webpack_1.webpackTemplate)({ port }));
    await (0, promises_1.writeFile)(pathFor('.babelrc'), (0, babel_1.babelTemplate)());
    await (0, promises_1.writeFile)(pathFor('package.json'), (0, package_1.packageTemplate)({
        name: snakeCaseName,
        spectacleVersion
    }));
};
exports.writeWebpackProjectFiles = writeWebpackProjectFiles;
/**
 * Generate a vite-based project
 */
const writeViteProjectFiles = async (options) => {
    const { snakeCaseName, spectacleVersion, port } = options;
    const { pathFor } = await prepForProjectWrite(options);
    await (0, promises_1.writeFile)(pathFor('package.json'), (0, package_1.vitePackageTemplate)({
        name: snakeCaseName,
        spectacleVersion,
        port
    }));
    await (0, promises_1.writeFile)(pathFor(`vite.config.ts`), (0, viteConfig_1.viteConfigTemplate)());
};
exports.writeViteProjectFiles = writeViteProjectFiles;
/**
 * Generate a one-page project
 */
const writeOnePageHTMLFile = async ({ snakeCaseName, name, lang }) => {
    await (0, promises_1.writeFile)(path_1.default.resolve(process.cwd(), `${snakeCaseName}.html`), (0, one_page_1.createOnePage)(name, lang));
};
exports.writeOnePageHTMLFile = writeOnePageHTMLFile;
